/* @(#)src/common/kernel/base/vx_disklog.h	3.17.10.2 09 Jun 1998 15:17:14 -  */
/* #ident "@(#)vxfs:src/common/kernel/base/vx_disklog.h	3.17.10.2" */

/*
 * Copyright (c) 1997 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 *
 * 		RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 * 		VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef	_FS_VXFS_VX_DISKLOG_H
#define	_FS_VXFS_VX_DISKLOG_H

/*
 * The structures in the vxfs log.
 */

/*
 * The log format version number for this kernel.
 *
 * Whenever new transaction types are introduced or in some other
 * way the contents of the vxfs log are made different from previous
 * releases, we establish a new VX_LOGVERSION.  mkfs writes this into
 * the super-block of every file system and fsck replay consults the
 * version to know whether or not it is capable of replaying the log
 * contents of a dirty file system.  The kernel mount code will write
 * back its own VX_LOGVERSION to a file system that is successfully
 * mounted.
 *
 * A log version is different from a file system layout version (e.g.
 * VX_VERSION1, VX_VERSION2, etc).  The log version can change across
 * various releases of a given layout version.  The implication is that
 * a new release of vxfs cannot necessarily deal with the log contents
 * of existing file systems written by the previous release of vxfs.
 * File systems should be cleanly unmounted before installation of a
 * new vxfs release.
 */

#define	VX_LOGVERSION	9

/*
 * The structure of the generic vxfs log record.
 *
 * Each log record is variable length and consists of 1 or more functions
 * constituting a transaction.  Each function in the transaction has the
 * same log_id value.  The log_ser value is incremented for each function.
 *
 * The log record for a function is always a multiple of 16 bytes and does
 * not cross a VX_ATOMIC_IOSZ boundary.  The log_len is always the exact
 * number of data bytes, and does not include padding.  If the dependent
 * data will cause a VX_ATOMIC_IOSZ boundary to be crossed, a continuation
 * record is used.  The continuation records have the same log_id, log_func,
 * log_ser fields, but have an incrementing log_fser field.  Since many
 * log records are short, one log block may contain multiple transactions.
 *
 * log_id is the transaction id number, which is an incrementing serial
 * number held in the file system's fs_logid.  If the id wraps to zero,
 * it is incremented to one, so an id of zero indicates nothing there.
 * There should never be a missing number in the log.  A change in log_id
 * indicates that a new transaction has begun.
 *
 * log_func   is the function type as defined in vx_tran.h
 * log_ser    is the function number within the transaction (0-127)
 * log_lser   is the last function number within the transaction
 * log_fser   is the continuation record number within the function (0-127)
 * log_lfser  is the last continuation record number within the function
 *
 * log_len is the number of data bytes in the record.  If a function has
 * continuation records, the number of data bytes in the function is the
 * sum of the log_len fields.  Note that records are always a multiple of
 * 16 bytes, but the log_len field is always exact.
 */

#define	VX_LOGFILL	8			/* round to 32 bytes */

struct vx_log {
	long		log_vxldidl;		/* sector identification */
	long		log_vxldid2;		/* sector identification */
	long		log_vxldid3;		/* sector identification */
	long		log_id;			/* transaction id number */
	short		log_func;		/* function */
	char		log_ser;		/* serial # within trans */
	char		log_lser;		/* last serial # within trans */
	char		log_fser;		/* serial # within func */
	char		log_lfser;		/* last serial # within func */
	short		log_len;		/* data length */
	char		log_depend[VX_LOGFILL];	/* variable */
};

#define	VX_LOGMASK	(sizeof (struct vx_log) - 1)
#define	VX_LOGOVER	(sizeof (struct vx_log) - VX_LOGFILL)
#define	VX_LOGMAX	(VX_ATOMIC_IOSZ - VX_LOGOVER)

#define	VX_LOGROUNDUP(len)	\
		(((len) + VX_LOGOVER + VX_LOGMASK)  & ~VX_LOGMASK)

/*
 * transaction function ids
 */

#define	VX_IMTRAN	1		/* inode modification */
#define	VX_DATRAN	2		/* add directory entry */
#define	VX_DACTRAN	3		/* add and compress directory */
#define	VX_DRTRAN	4		/* remove directory entry */
#define	VX_IETRAN	5		/* IMAP or IEMAP operation */
#define	VX_EMTRAN	6		/* extent map */
#define	VX_TRTRAN	7		/* indirect truncation */
#define	VX_WLTRAN	8		/* write a symlink */
#define	VX_TATRAN	9		/* indirect allocation */
#define	VX_LWRTRAN	10		/* logged write subfunction */
#define	VX_LWRDATA	11		/* logged write data */
#define	VX_LWRDONE	12		/* logged writes done */
#define	VX_CUTRAN	13		/* CUT change transaction */
#define	VX_FSHDTRAN	14		/* fshead change transaction */
#define	VX_ILTRAN	15		/* ilist change transaction */
#define	VX_LCTRAN	16		/* link count table update */
#define	VX_LBTRAN	17		/* logged buffer changes */
#define	VX_LBDATA	18		/* logged buffer data */
#define	VX_BSDQTRAN	19		/* bsd quota change transaction */
#define	VX_MULTIDONE	103		/* group done record */

#define	VX_ENTMAX	32		/* max ops in a map subfunction */

struct vx_emlog {
	int	log_maptype;		/* emap or smap */
	long	log_sdevid;		/* summary device id */
	daddr_t	log_sbno;		/* summary block number */
	long	log_mdevid;		/* map device id */
	daddr_t	log_mbno;		/* map starting block */
	long	log_edevid;		/* extent device id */
	struct vx_ement {
		long	em_op;		/* VX_EMALLOC or VX_EMFREE */
		daddr_t	em_ebno;	/* extent starting block */
		long	em_elen;	/* extent length */
	} log_ent[VX_ENTMAX];
};

#define	VX_EMALLOC	1		/* extent allocation */
#define	VX_EMFREE	2		/* extent free */

/*
 * Map types as used in log_maptype.
 *
 * These are also used in the mlink incore structure fm_type.
 */

#define	VX_EMAP		1	/* free extent map */
#define	VX_IMAP		2	/* free inode map */
#define	VX_IEMAP	3	/* extended inode operations map */
#define	VX_SMAP		4	/* eau state map */

/*
 * inode map update subfunction
 */

struct vx_ielog {
	long	iel_maptype;			/* IMAP or IEMAP */
	long	iel_fset;			/* fileset index */
	long	iel_ilist;			/* inode list type */
	long	iel_devid;			/* map device id */
	daddr_t	iel_bno;			/* map block number */
	long	iel_aun;			/* au number */
	long	iel_ausdevid;			/* au summmary device id */
	daddr_t	iel_ausbno;			/* au summmary bno */
	struct vx_iment {
		vx_ino32_t	iel_ino;	/* inode number */
		long		iel_op;		/* set or clear */
	} log_ent[VX_ENTMAX];
};

/*
 * iel_op values
 */

#define	VX_IASET	1		/* bit set */
#define	VX_IACLR	2		/* bit clear */

/*
 * inode modification transaction
 */

struct vx_imlog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode number */
	long		log_devid;	/* device id */
	long		log_pad;	/* for 8 byte alignment on osize */
	daddr_t		log_bno;	/* block number */
	vxhyper_t	log_osz;	/* saved value of i_nsize */
	char		log_inodes[4];	/* new and old inodes (variable) */
};

#define	log_osize		log_osz.vh_value

/*
 * directory entry addition transaction
 */

struct vx_dalog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* directory inode */
	long		log_devid;	/* directory block device id */
	daddr_t		log_bno;	/* directory block no */
	long		log_boff;	/* offset in block */
	long		log_blen;	/* length of block */
	struct vx_mindirect log_old;	/* old directory entry */
	struct vx_direct log_new;	/* new directory entry */
};

/*
 * directory entry removal transaction
 */

struct vx_drlog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* directory inode */
	long		log_devid;	/* directory block device id */
	daddr_t		log_bno;	/* directory block no */
	long		log_poff;	/* offset in block of prev */
	long		log_roff;	/* offset in block of rem */
	long		log_blen;	/* length of block */
	struct vx_mindirect log_prev;	/* previous directory entry */
	struct vx_direct log_rem;	/* removed directory entry */
};

/*
 * if VX_DACTRAN, then the log area is an vx_daclog structure followed by
 * the before and after contents of the entire block
 */

struct vx_daclog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* directory inode */
	long		log_devid;	/* directory block device id */
	daddr_t		log_bno;	/* directory block no */
	long		log_oldblen;	/* length of pre-image block */
	long		log_newblen;	/* length of post-image block */
};

/*
 * Used to log the pathname if the symbolic link doesn't go
 * into immediate data.
 */

struct vx_wltran {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode containing extent */
	long		log_devid;	/* symlink block device id */
	daddr_t		log_bno;	/* symlink block no */
	long		log_len;	/* length of extent */
	long		log_dlen;	/* length of data */
	char		log_data[4];	/* symlink data */
};

/*
 * Logged writes on a file are done.
 */

struct vx_lwrdonelog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode number */
};

#define	VX_MAXTRTRAN	6		/* Max trtrans in a truncate */

/*
 * indirect extent truncation
 */

struct vx_trlog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode number */
	int		log_lvl;	/* indirection level */
	long		log_adevid;	/* devid of address extent */
	daddr_t		log_abno;	/* bno of address extent */
	struct vx_trent {
		long	log_aoff;	/* offset in address extent */
		daddr_t	log_ebno;	/* bno of data extent */
	} log_ent[VX_ENTMAX];
};

/*
 * indirect extent addition
 *
 * VX_MAXALLOC is an arbitrary limit on the size of a transaction, to
 * avoid letting a transaction get infinitely large.
 */

#define	VX_MAXALLOC	50		/* max allocations per tran */

struct vx_talog {
	int		log_new;	/* set if addr extent to be cleared */
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode number */
	int		log_lvl;	/* indirection level */
	long		log_adevid;	/* device id address extent */
	daddr_t		log_abno;	/* bno of address extent */
	struct vx_taent {
		long	log_aind;	/* index in address extent */
		daddr_t	log_ebno;	/* bno of data extent */
	} log_taent[VX_MAXALLOC];
};

/*
 * Fileset header modification subfunction.
 */

struct vx_fslog {
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode number */
	daddr_t		log_devid;	/* device id */
	daddr_t		log_bno;	/* block number */
	long		log_padding;	/* 8 byte align structure */
	union vx_fsethead log_new;	/* new fileset header */
};

/*
 * Done and undo subfunction information.
 */

struct vx_duclog {
	int	duc_logid;		/* log id of failed transaction */
	int	duc_err;		/* error code */
};

/*
 * CUT modification subfunction.
 */

struct vx_culog {
	union vx_cuent	log_new;	/* new cut entry */
	int		log_offset;	/* index of entry in CUT */
	long		log_devid;	/* device id of CUT block */
	daddr_t		log_bno;	/* block number of CUT block */
};

/*
 * link count table update subfunction
 */

struct vx_lclog {
	long	log_fset;		/* fileset index */
	long	log_devid;		/* lct device id */
	daddr_t	log_bno;		/* lct block number */
	long	log_padding;		/* 8 byte align structure */
	struct vx_lcent {
		union vx_lctdisk lcl_new;   /* new value */
		union vx_lctdisk lcl_old;   /* old value */
		vx_ino32_t	 lcl_ino;   /* inode number */
		int		 lcl_delta; /* change in value */
		int		 lcl_extop; /* IEREMOVE set on file */
	} log_ent[VX_ENTMAX];
};

/*
 * logged write subfunction.  This is followed in
 * the log by a logged write data subfunction.
 */

struct vx_lwrlog {
	long		log_devid;	/* device id */
	long		log_fset;	/* fileset index */
	long		log_ilist;	/* inode list type */
	vx_ino32_t	log_ino;	/* inode being written */
	vxhyper_t	log_foffst;	/* offset of logged write in file */
	int		log_length;	/* length of changes */
};

#define	log_foffset		log_foffst.vh_value

/*
 * logged buffer changes subfunction.  This is followed in
 * the log by a logged buffer data subfunction.
 */

struct vx_lblog {
	long	log_devid;	/* device id */
	daddr_t	log_bno;	/* block number */
	int	log_buflen;	/* buffer length */
	int	log_created;	/* buffer created, replay must clear */
	int	log_offset;	/* offset in buffer of first change */
	int	log_length;	/* length of changes */
};

/*
 * BSD quota modification subfunction.
 */

struct vx_bsdqlog {
	long	log_fset;	/* fset owning quota */
	int	log_type;	/* type of quota */
	ulong	log_id;		/* uid or gid for quota entry */
	long	log_devid;	/* device id of log_bno */
	daddr_t	log_bno;	/* block containing quota entry */
	long	log_padding;	/* 8 byte align structure */
	struct vx_qrec log_new;	/* new quota file entry */
};

/*
 * Minimum log size for disk layout version 4.
 */

#define VX_MIN_LOGSIZE_V4	256	/* V4 minimum log size in K bytes */

#endif	/* _FS_VXFS_VX_DISKLOG_H */
